package fr.openium.consentium_ui.ui

import android.content.Context
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import dagger.hilt.android.lifecycle.HiltViewModel
import dagger.hilt.android.qualifiers.ApplicationContext
import fr.openium.consentium_ui.R
import fr.openium.consentium_ui.domain.repository.ConsentiumRepository
import fr.openium.consentium_ui.domain.repository.ConsentiumUIRepositoryResponse
import fr.openium.consentium_ui.domain.usecase.GetApplicationLanguageUseCase
import fr.openium.consentium_ui.domain.usecase.GetConfigTextForLanguageUseCase
import fr.openium.consentium_ui.domain.usecase.GetConfigTextForLanguageUseCaseResponse
import fr.openium.consentium_ui.ui.adapter.toDetailConsentUI
import fr.openium.consentium_ui.ui.adapter.toGeneralConsentUI
import fr.openium.consentium_ui.ui.state.ConsentiumUIState
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.launch
import javax.inject.Inject

@HiltViewModel
internal class ConsentiumUIViewModel @Inject constructor(
    @ApplicationContext private val context: Context,
    private val consentiumUIRepository: ConsentiumRepository,
    private val configTextLanguageUseCase: GetConfigTextForLanguageUseCase,
    private val getApplicationLanguageUseCase: GetApplicationLanguageUseCase,
) : ViewModel() {

    private val _state = MutableStateFlow<ConsentiumUIState>(ConsentiumUIState.Loading)
    val state: StateFlow<ConsentiumUIState> by lazy { _state.asStateFlow() }

    fun init(appId: String, apiKey: String) {
        viewModelScope.launch {

            _state.value = ConsentiumUIState.Loading

            when (val consentiumUIFetchResponse = consentiumUIRepository.getConsentiumConfig(appId, apiKey)) {

                is ConsentiumUIRepositoryResponse.Success -> {
                    val consentUIResponse =
                        configTextLanguageUseCase(getApplicationLanguageUseCase(context), consentiumUIFetchResponse.contentConfigData)

                    when (val consentUI = consentUIResponse) {

                        is GetConfigTextForLanguageUseCaseResponse.Success -> {
                            _state.emit(
                                ConsentiumUIState.Loaded(
                                    generalConsentUI = consentUI.configData.toGeneralConsentUI(),
                                    detailConsentUI = consentUI.configData.toDetailConsentUI(),
                                )
                            )
                        }

                        is GetConfigTextForLanguageUseCaseResponse.DefaultLanguage -> {
                            _state.emit(
                                ConsentiumUIState.Loaded(
                                    generalConsentUI = consentUI.configData.toGeneralConsentUI(),
                                    detailConsentUI = consentUI.configData.toDetailConsentUI(),
                                )
                            )
                        }

                        is GetConfigTextForLanguageUseCaseResponse.Error -> {
                            _state.emit(ConsentiumUIState.Error(context.getString(R.string.consents_error_loading_consents)))
                        }

                    }
                }

                is ConsentiumUIRepositoryResponse.Error -> {
                    _state.value = ConsentiumUIState.Error(context.getString(R.string.consents_error_loading_consents))
                }

            }
        }
    }
}